/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is NSS-GUI code.
 *
 * The Initial Developer of the Original Code is
 * Red Hat, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Kai Engert <kengert@redhat.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <boost/program_options/cmdline.hpp>
#include <boost/program_options/options_description.hpp>
#include <boost/program_options/variables_map.hpp>
#include <boost/program_options/parsers.hpp>
#include <boost/program_options/value_semantic.hpp>
#include <boost/algorithm/string/trim.hpp>
#include <boost/cstdlib.hpp>
namespace po = boost::program_options;
#include <string>
#include <fstream>
#include <iostream>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
using std::cout;
using std::string;

int main(int argc, char *argv[])
try {
  const char *opt_dir = "dbdir";
  const char *opt_ini = "ini";
  const char *env_name = "MOZPSM_NSSDBDIR_OVERRIDE";

  po::options_description desc("NSS GUI parameters");
  desc.add_options()
      ("help", "show help message")
      (opt_dir, po::value<string>(), "NSS database directory")
      (opt_ini, po::value<string>(), "Path to xrnssgui.ini")
      ;

  po::variables_map vm;
  try {
    po::store(po::parse_command_line(argc, argv, desc), vm);
    po::notify(vm);

    if (vm.count("help")) {
        cout << desc << "\n";
        return boost::exit_failure;
    }
  } catch (boost::program_options::unknown_option const &e) {
    cout << "error: " << e.what() << "\n";
    cout << desc << "\n";
    return boost::exit_failure;
  }

  if (vm.count(opt_dir)) {
    // use given dbdir, overwrite env var
    string dbdir = vm[opt_dir].as<string>();
    cout << env_name << "=" << dbdir << "\n";
    setenv(env_name, dbdir.c_str(), true);
  } else {
    // do not overwrite existing env var
    string environment_override;
    const char *getenv_result = getenv(env_name);
    if (getenv_result) {
      environment_override = getenv_result;
    }
    if (environment_override.empty()) {
      // no environment varilable, set default path
      string db_directory;
      bool use_system_db = false;

      uid_t euid = geteuid();
      if (euid == 0) {
        // we are running with root permissions, always default to system db
        use_system_db = true;
      }
      else {
        // for non-root use system db only if libnsssysinit is enabled
        std::ifstream f;
        f.open("/etc/pki/nssdb/pkcs11.txt");
        if (f.is_open()) {
          while (!f.eof()) {
            string line;
            getline(f, line);
            boost::algorithm::trim(line);
            if (line.empty())
              continue;
            if (line[0] == '#')
              continue;
            if (line == "library=libnsssysinit.so") {
              use_system_db = 1;
              break;
            }
          }
          f.close();
        }

        if (!use_system_db) {
          getenv_result = getenv("HOME");
          if (!getenv_result) {
            cout << "environment variable HOME not set, cannot use automatic dbdir\n";
            return 1;
          }
          db_directory = getenv_result;
          db_directory += "/.pki/nssdb/";
        }
      }

      if (use_system_db) {
        db_directory = "/etc/pki/nssdb/";
      }

      struct stat stat_info;
      int stat_result = stat(db_directory.c_str(), &stat_info);
      if (stat_result != 0) {
        cout << "error: default directory " << db_directory << " does not exist or can not be accessed.\n";
        cout << "You should either create this path or use the --dbdir parameter.\n";
        return 1;
      }

      string nssdb = "sql:" + db_directory;
      cout << env_name << "=" << nssdb << "\n";
      setenv(env_name, nssdb.c_str(), false);
    }
    else {
      cout << env_name << "=" << environment_override << "\n";
    }
  }

  const char *ini_path;
  if (vm.count(opt_ini)) {
    ini_path = vm[opt_ini].as<string>().c_str();
  } else {
    ini_path = "/usr/share/nss-gui/xrnssgui.ini";
  }

  string cmd = "xulrunner ";
  cmd += ini_path;
  system(cmd.c_str());

  return boost::exit_success;
} catch ( std::exception const &e ) {
  cout << "error: " << e.what() << "\n";
  return boost::exit_failure;
}
